///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
//                IssBuilder             //
////////////////////////////////////////////////////////////

unit IssBuilderServiceClass; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, ExtraFunctionsLcl, GGConsUtilServiceClass, GGBuildToolsShared, ExtraFileUtilsLCL,
  IssBuilderLogic;
  
const
  GGBT_IssBuilderAppName = ' - Inno Setup builder';
  
type
  TIssBuilderServiceClass = class (TGGConsUtilServiceClass)       //      IssBuilder
  private
  protected
    LogicClass : TIssBuilderLogic;

    // .
    procedure PostExternalMessage(const TextMessage : AnsiString);    //     .
  public
    //
    SconfFilename : AnsiString;

    //-...
    constructor Create; override;
    destructor Destroy; override;

    // ...
    procedure ProcessParameters;  override;     //   .
    procedure DoWork;        override;          //   %).
    procedure ShowDescription; override;        //  .
    procedure ShowOptions;  override;           //    .
    procedure ProcessExtraSconfParams;          // ,   sconf.
  end;

var
  IssBuilderServClass : TIssBuilderServiceClass;

implementation

/////////////////////////////////////////////
//         TIssBuilderServiceClass         //
/////////////////////////////////////////////

//-----------------------------------------//
//        -...      //
//-----------------------------------------//

constructor TIssBuilderServiceClass.Create;
begin
  //  ...
  SconfFilename := '';

  //   ...
  LogicClass := TIssBuilderLogic.Create;

  //  .
  Self.GGConsAppName := GGBT_IssBuilderAppName;

  //  ...
  inherited;
end;

destructor TIssBuilderServiceClass.Destroy;
begin
  // 
  LogicClass.Free;

  //  
  inherited;
end;

//------------------------------------------//
//            ...           //
//------------------------------------------//

procedure TIssBuilderServiceClass.PostExternalMessage(const TextMessage : AnsiString);
//     .
begin
  Writeln(TextMessage);
end;

//------------------------------------------//
//             ...            //
//------------------------------------------//

procedure TIssBuilderServiceClass.ProcessParameters;
//   .
begin
  //  
  inherited;

  if Application.HasOption('filepath') then begin
    //   .
    Self.SconfFilename := Application.GetOptionValue('filepath');
  end;
  
  if Application.HasOption('nocachewrite') or Application.HasOption('n') then begin
    //  -  .
    Self.LogicClass.NoCacheWrite := true;
  end;
  
  //,    sconf  .    DoWork().
end;

procedure TIssBuilderServiceClass.DoWork;
//   %).
var
  AllOk : Boolean;
begin
  //     -    ProcessFurther := false
  //    .

  //  
  inherited;
  
  // .
  AllOk := true;

  //       -    ....
  if ProcessFurther = true then begin
    //  
    ProcessFurther := false;
    //....
    if Application.HasOption('filepath') = true then begin
      //   .
      //  ...
      Self.ShowVersion();
      Writeln('');
      //    .
      Self.SconfFilename := ResolveLocalPath(Self.SconfFilename,GetCurrentDir);

      //        .    .
      if FileExists(Self.SconfFilename) = true then begin
        //.
        
        // Sconf.
        Writeln('Reading sconf from '+Self.SconfFilename);
        AllOk := Self.LogicClass.ProjectSconf.Load(Self.SconfFilename);
        Self.LogicClass.SconfFilePath := Self.SconfFilename;
        
        if AllOk = true then begin
          //.
          Writeln('Readed.');
          
          //     sconf     ,     .
          Self.ProcessExtraSconfParams;
          
          //. ,  ,  ,      .
          Self.LogicClass.PostExternalMessage := @Self.PostExternalMessage;
          AllOk := Self.LogicClass.ProcessSconf();
          
          //  -       .
          if AllOk = true then begin
            //.
            Writeln('Done.');
            Writeln('');
          end
          else begin
            // - .
            Writeln('Error. IssBuilding stopped.');
            Writeln('');
          end;
        end;
        
      end
      else begin
        //.   .
        Writeln('Error! Project sconf file is not exists! : ');
        Writeln(Self.SconfFilename);
        Writeln('');
      end;
    end
    else begin
      //.       .
      Self.ShowHelp();
      Writeln('Error!!! "--filepath=<value>" option is required!');
      Writeln('');
    end;
  end;

end;

procedure TIssBuilderServiceClass.ShowDescription;
//  .
begin
  inherited;
  Writeln('Description:');
  Writeln('             Generates *.iss Inno Setup scripts for your program''s installs,');
  Writeln('             then launching Inno Setup compiler to create an installation');
  Writeln('             package.');
end;

procedure TIssBuilderServiceClass.ShowOptions;
//    .
begin
  //  ...
  inherited;

  //     %).
  Writeln('          --filepath=<value> : sets path to the project''s configuration file.');
  Writeln('          -n, --nocachewrite : don''t save anything into cache, ignoring');
  Writeln('            project''s configuration file options.');
  Writeln('          ');
  Writeln('Options to redeclare [IssBuilder] section''s elements from project''s');
  Writeln('configuration file:');
  Writeln('          ');
  Writeln('          --FilesDir=<value> : path (relative to project''s root or absolute)');
  Writeln('            to directory with files for installation. All paths may be relative');
  Writeln('            to project''s root or they may be absolute.');
  Writeln('          --DefaultIssName=<value> : name of default *.iss file.');
  Writeln('          --OutputDir=<value> : directory, where to put compiled installs.');
  Writeln('          --VersionsCacheDir=<value> : directory for versions cache.');
  Writeln('          --SaveToCache=<value> : value is 0 or 1. If 1 - saves all version''s');
  Writeln('            files into cache (from FilesDir), not only *.iss and *.flist files.');
  Writeln('          --CreateFull=<value> : value is 0 or 1. If 1 - creates a full');
  Writeln('            installation package (full version and may be updates).');
  Writeln('          --CreateFull_AllowUpdate=<value> : value is 0 or 1. If 1 - gives full');
  Writeln('            installation an ability to update previous versions without they ');
  Writeln('            uninstallation.');
  Writeln('          --CreateFull_UpdVers=<value> : What previous versions to make');
  Writeln('            updatable - if value is "All" - all previous versions which are');
  Writeln('            exists in a cache will be updatable. If value is "From:X", where X');
  Writeln('            is a ver number (4 nubers, delimited by dots) - all versions from');
  Writeln('            X (including it), which are exists in a cache will be updatable. If');
  Writeln('            value is a number - last value number of versions, which are exists');
  Writeln('            in a cache will be updatable.');
  Writeln('          --CreateSeparateUpds=<value> : value is 0 or 1. If 1 - separate');
  Writeln('            update installs, which can only update a speciefic one version');
  Writeln('            will be created.');
  Writeln('          --CreateSeparateUpds_UpdVers=<value> : same as CreateFull_UpdVers.');
  Writeln('          --CreateCumulateUpdate=<value> : value is 0 or 1. If 1 - cumulative');
  Writeln('            update install, which can update a number of different versions');
  Writeln('            will be created.');
  Writeln('          --CreateCumulateUpdate_UpdVers=<value> : same as CreateFull_UpdVers.');
end;

procedure TIssBuilderServiceClass.ProcessExtraSconfParams;
// ,   sconf.
begin

  if Application.HasOption('FilesDir') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.FilesDir := Application.GetOptionValue('FilesDir');
  end;
  
  if Application.HasOption('DefaultIssName') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.DefaultIssName := Application.GetOptionValue('DefaultIssName');
  end;
  
  if Application.HasOption('OutputDir') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.OutputDir := Application.GetOptionValue('OutputDir');
  end;
  
  if Application.HasOption('VersionsCacheDir') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.VersionsCacheDir := Application.GetOptionValue('VersionsCacheDir');
  end;
  
  if Application.HasOption('SaveToCache') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.SaveToCache := StrToBool(Application.GetOptionValue('SaveToCache'));
  end;
  
  if Application.HasOption('CreateFull') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.CreateFull := StrToBool(Application.GetOptionValue('CreateFull'));
  end;
  
  if Application.HasOption('CreateFull_AllowUpdate') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.CreateFull_AllowUpdate := StrToBool(Application.GetOptionValue('CreateFull_AllowUpdate'));
  end;
  
  if Application.HasOption('CreateFull_UpdVers') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.CreateFull_UpdVers := Application.GetOptionValue('CreateFull_UpdVers');
  end;
  
  if Application.HasOption('CreateSeparateUpds') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.CreateSeparateUpds := StrToBool(Application.GetOptionValue('CreateSeparateUpds'));
  end;
  
  if Application.HasOption('CreateSeparateUpds_UpdVers') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.CreateSeparateUpds_UpdVers := Application.GetOptionValue('CreateSeparateUpds_UpdVers');
  end;
  
  if Application.HasOption('CreateCumulateUpdate') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.CreateCumulateUpdate := StrToBool(Application.GetOptionValue('CreateCumulateUpdate'));
  end;
  
  if Application.HasOption('CreateCumulateUpdate_UpdVers') then begin
    Self.LogicClass.ProjectSconf.IssBuilderSection.CreateCumulateUpdate_UpdVers := Application.GetOptionValue('CreateCumulateUpdate_UpdVers');
  end;

end;

/////////////////////////////////
initialization

// %).  ,   ...     %)
begin
  IssBuilderServClass := TIssBuilderServiceClass.Create;
end;

finalization
//  ...
begin
  IssBuilderServClass.Free;
end;

end.
